/**
Vector container
http://en.cppreference.com/w/cpp/container/vector
http://www.cplusplus.com/reference/vector/vector/
**/

#include<iostream>
#include<vector>
#include <algorithm>

using namespace std;

int main()
{
    // A container (arraylist)
    vector<int> v1{3, 8, 65};
    vector<int> v2;
    v2 = v1;
    vector<int> v{v2};

    if(v == v2)
    {
        cout << "The vectors are equal" << endl;
    }

    // Pre-built operations
    v.push_back(4);
    v.push_back(54);
    v.push_back(3);
    v.push_back(2);
    v.push_back(1);
    v.push_back(4);
    v.push_back(5);
    v.at(4) = 123;
    v[3] = 6;
    //v[30] = 4; // does this work?
    //v.at(30) = 4;

    cout << "Size is: " << v.size() << endl;

    // Old iteration method
    for(int i = 0; i < v.size(); i++)
    {
        cout << v[i] << " ";
    }
    cout << endl;

    for(int i : v)
    {
        cout << i << " ";
    }
    cout << endl;

    // declare a read-only iterator
    vector<int>::const_iterator it;

    // assign it to the start of the vector
    it = v.cbegin();

    // while it hasn't reach the end
    while(it != v.end())
    {
        // print the value of the element it points to
        cout << *it << " ";

        // and iterate to the next element
        ++it;
    }

    // Finding an element is more useful when it's multipart
    // Why *()?
    cout << endl << "Find 1? "<< *(find(v.begin(), v.end(), 1));

    // Can specify sub-ranges with iterators
    cout << endl << "What is the min? " << *min_element(v.begin(), v.end()) << endl;

    // Sort the array in a range
    sort(v.begin(), v.end());

    // Algorithms operate on containers using iterators
    cout << endl << "Is 3 present? " << *find(v.begin(), v.end(), 3 ) << endl;

    // can also use a for loop with iterators
    for(vector<int>::iterator i = v.begin(); i != v.end(); i++)
    {
        cout << *i << " ";
    }

    // Random access iterator
    vector<int>::iterator temp = v.begin();
    v.insert(temp + 4, 23);

    // Remove last element
    v.pop_back();

    cout << endl;

    for(int i : v)
    {
        cout << i << " ";
    }

    // Can erase entire ranges using iterators
    v.erase(v.begin(), v.end() - 2);

    cout << endl;

    for(vector<int>::iterator i = v.begin(); i != v.end(); i++)
    {
        cout << *i << " ";
    }

    cout << endl;

    // don't need to bother writing type (why auto is nice)
    for(auto i = v.begin(); i != v.end(); i++)
    {
        cout << *i++ << " ";
    }

    cout << endl;

    // const version
    for(auto i = v.cbegin(); i != v.end(); i++)
    {
        cout << *i << " ";
    }

    cout << endl;

    v.clear();

    cout << "Size is: " << v.size() << endl;

    // Two dimensional array -- easy!
    // Alternate construction (number, element to be repeated)
    cout << endl << "Making a 2D array: " << endl;
    vector< vector<int> > Matrix(4, vector<int>(10, -1));

    for(int j = 0; j < Matrix.size(); j++)
    {
        for(int k = 0; k < Matrix[j].size(); k++)
        {
            cout << Matrix[j][k];
        }
        cout << endl;
    }

    // What about memory management??

    // How do we do traverse the 2D with iterators??


    return 0;
}
